@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2006-2007  Nokia Corporation
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -57,6 +57,7 @@
 #include "dbus-common.h"
 #include "agent.h"
 #include "storage.h"
+#include "dbus-hci.h"
 
 static DBusConnection *connection = NULL;
 
@@ -239,6 +240,10 @@ int hcid_dbus_request_pin(int dev, bdaddr_t *sba, struct hci_conn_info *ci)
 		return -1;
 	}
 
+	if (!adapter_pairing_initiator(adapter, &ci->bdaddr) &&
+			!adapter_is_pairable(adapter))
+		return -EPERM;
+
 	ba2str(&ci->bdaddr, addr);
 
 	device = adapter_find_device(adapter, addr);
@@ -249,8 +254,10 @@ int hcid_dbus_request_pin(int dev, bdaddr_t *sba, struct hci_conn_info *ci)
 	if (!agent)
 		agent = adapter_get_agent(adapter);
 
-	if (!agent)
+	if (!agent) {
+		error("No agent available for PIN request");
 		return -EPERM;
+	}
 
 	if (!device) {
 		device = adapter_create_device(connection, adapter, addr);
@@ -503,7 +510,7 @@ int hcid_dbus_user_passkey(bdaddr_t *sba, bdaddr_t *dba)
 		agent = adapter_get_agent(adapter);
 
 	if (!agent) {
-		error("No agent available for user confirm request");
+		error("No agent available for user passkey request");
 		return -1;
 	}
 
@@ -877,9 +884,7 @@ static void emit_device_found(const char *path, const char *address,
 	append_dict_valist(&iter, first_key, var_args);
 	va_end(var_args);
 
-	dbus_connection_send(connection, signal, NULL);
-
-	dbus_message_unref(signal);
+	g_dbus_send_message(connection, signal);
 }
 
 void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
@@ -888,9 +893,11 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 	char filename[PATH_MAX + 1];
 	struct btd_adapter *adapter;
 	char local_addr[18], peer_addr[18], *alias, *name, *tmp_name;
+	const char *real_alias;
 	const char *path, *icon, *paddr = peer_addr;
 	struct remote_dev_info *dev, match;
 	dbus_int16_t tmp_rssi = rssi;
+	dbus_bool_t legacy = TRUE;
 	uint8_t name_type = 0x00;
 	name_status_t name_status;
 	int state;
@@ -906,8 +913,10 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 
 	write_remote_class(local, peer, class);
 
-	if (data)
+	if (data) {
 		write_remote_eir(local, peer, data);
+		legacy = FALSE;
+	}
 
 	/*
 	 * workaround to identify situation when the daemon started and
@@ -944,12 +953,15 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 	name = textfile_get(filename, peer_addr);
 
 	if (!alias) {
+		real_alias = NULL;
+
 		if (!name) {
 			alias = g_strdup(peer_addr);
 			g_strdelimit(alias, ":", '-');
 		} else
 			alias = g_strdup(name);
-	}
+	} else
+		real_alias = alias;
 
 	tmp_name = extract_eir_name(data, &name_type);
 	if (tmp_name) {
@@ -972,32 +984,25 @@ void hcid_dbus_inquiry_result(bdaddr_t *local, bdaddr_t *peer, uint32_t class,
 	path = adapter_get_path(adapter);
 	icon = class_to_icon(class);
 
-	if (name) {
-		if (name_type != 0x08)
-			name_status = NAME_SENT;
-
-		emit_device_found(path, paddr,
+	emit_device_found(path, paddr,
 				"Address", DBUS_TYPE_STRING, &paddr,
 				"Class", DBUS_TYPE_UINT32, &class,
 				"Icon", DBUS_TYPE_STRING, &icon,
 				"RSSI", DBUS_TYPE_INT16, &tmp_rssi,
 				"Name", DBUS_TYPE_STRING, &name,
-				"Alias", DBUS_TYPE_STRING, &alias, NULL);
+				"Alias", DBUS_TYPE_STRING, &alias,
+				"LegacyPairing", DBUS_TYPE_BOOLEAN, &legacy,
+				NULL);
 
-		g_free(name);
-	} else {
-		emit_device_found(path, paddr,
-				"Address", DBUS_TYPE_STRING, &paddr,
-				"Class", DBUS_TYPE_UINT32, &class,
-				"Icon", DBUS_TYPE_STRING, &icon,
-				"RSSI", DBUS_TYPE_INT16, &tmp_rssi,
-				"Alias", DBUS_TYPE_STRING, &alias, NULL);
-	}
-
-	g_free(alias);
+	if (name && name_type != 0x08)
+		name_status = NAME_SENT;
 
 	/* add in the list to track name sent/pending */
-	adapter_add_found_device(adapter, peer, rssi, name_status);
+	adapter_add_found_device(adapter, peer, rssi, class, real_alias,
+					name_status);
+
+	g_free(name);
+	g_free(alias);
 }
 
 void hcid_dbus_remote_class(bdaddr_t *local, bdaddr_t *peer, uint32_t class)
@@ -1037,8 +1042,9 @@ void hcid_dbus_remote_name(bdaddr_t *local, bdaddr_t *peer, uint8_t status,
 {
 	struct btd_adapter *adapter;
 	char srcaddr[18], dstaddr[18];
-	const gchar *dev_path;
 	int state;
+	struct btd_device *device;
+	struct remote_dev_info match, *dev_info;
 
 	adapter = manager_find_adapter(local);
 	if (!adapter) {
@@ -1049,27 +1055,58 @@ void hcid_dbus_remote_name(bdaddr_t *local, bdaddr_t *peer, uint8_t status,
 	ba2str(local, srcaddr);
 	ba2str(peer, dstaddr);
 
-	if (!status) {
-		struct btd_device *device;
+	if (status != 0)
+		goto proceed;
 
-		device = adapter_find_device(adapter, dstaddr);
-		if (device) {
-			char alias[248];
+	bacpy(&match.bdaddr, peer);
+	match.name_status = NAME_ANY;
 
-			dev_path = device_get_path(device);
+	dev_info = adapter_search_found_devices(adapter, &match);
+	if (dev_info) {
+		const char *adapter_path = adapter_get_path(adapter);
+		const char *icon = class_to_icon(dev_info->class);
+		const char *alias, *paddr = dstaddr;
+		dbus_int16_t rssi = dev_info->rssi;
+		dbus_bool_t legacy;
 
-			emit_property_changed(connection, dev_path,
-						DEVICE_INTERFACE, "Name",
-						DBUS_TYPE_STRING, &name);
+		if (dev_info->alias)
+			alias = dev_info->alias;
+		else
+			alias = name;
 
-			if (read_device_alias(srcaddr, dstaddr,
-						alias, sizeof(alias)) < 1)
-				emit_property_changed(connection, dev_path,
-						DEVICE_INTERFACE, "Alias",
-						DBUS_TYPE_STRING, &name);
-		}
+		if (read_remote_eir(local, peer, NULL) < 0)
+			legacy = TRUE;
+		else
+			legacy = FALSE;
+
+		emit_device_found(adapter_path, dstaddr,
+				"Address", DBUS_TYPE_STRING, &paddr,
+				"Class", DBUS_TYPE_UINT32, &dev_info->class,
+				"Icon", DBUS_TYPE_STRING, &icon,
+				"RSSI", DBUS_TYPE_INT16, &rssi,
+				"Name", DBUS_TYPE_STRING, &name,
+				"Alias", DBUS_TYPE_STRING, &alias,
+				"LegacyPairing", DBUS_TYPE_BOOLEAN, &legacy,
+				NULL);
 	}
 
+	device = adapter_find_device(adapter, dstaddr);
+	if (device) {
+		char alias[248];
+		const char *dev_path = device_get_path(device);
+
+		emit_property_changed(connection, dev_path,
+				DEVICE_INTERFACE, "Name",
+				DBUS_TYPE_STRING, &name);
+
+		if (read_device_alias(srcaddr, dstaddr,
+					alias, sizeof(alias)) < 1)
+			emit_property_changed(connection, dev_path,
+					DEVICE_INTERFACE, "Alias",
+					DBUS_TYPE_STRING, &name);
+	}
+
+proceed:
 	/* remove from remote name request list */
 	adapter_remove_found_device(adapter, peer);
 
@@ -1208,8 +1245,7 @@ void hcid_dbus_disconn_complete(bdaddr_t *local, uint8_t status,
 		} else {
 			reply = new_authentication_return(bonding->msg,
 							HCI_AUTHENTICATION_FAILURE);
-			dbus_connection_send(connection, reply, NULL);
-			dbus_message_unref(reply);
+			g_dbus_send_message(connection, reply);
 		}
 		adapter_free_bonding_request(adapter);
 	}
@@ -1227,44 +1263,6 @@ void hcid_dbus_disconn_complete(bdaddr_t *local, uint8_t status,
 	}
 }
 
-int set_limited_discoverable(int dd, const uint8_t *cls, gboolean limited)
-{
-	uint32_t dev_class;
-	int num = (limited ? 2 : 1);
-	uint8_t lap[] = { 0x33, 0x8b, 0x9e, 0x00, 0x8b, 0x9e };
-	/*
-	 * 1: giac
-	 * 2: giac + liac
-	 */
-	if (hci_write_current_iac_lap(dd, num, lap, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
-		error("Can't write current IAC LAP: %s(%d)",
-				strerror(err), err);
-		return -err;
-	}
-
-	if (limited) {
-		if (cls[1] & 0x20)
-			return 0; /* Already limited */
-
-		dev_class = (cls[2] << 16) | ((cls[1] | 0x20) << 8) | cls[0];
-	} else {
-		if (!(cls[1] & 0x20))
-			return 0; /* Already clear */
-
-		dev_class = (cls[2] << 16) | ((cls[1] & 0xdf) << 8) | cls[0];
-	}
-
-	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
-		error("Can't write class of device: %s (%d)",
-							strerror(err), err);
-		return -err;
-	}
-
-	return 0;
-}
-
 int set_service_classes(int dd, const uint8_t *cls, uint8_t value)
 {
 	uint32_t dev_class;
@@ -1275,43 +1273,10 @@ int set_service_classes(int dd, const uint8_t *cls, uint8_t value)
 	dev_class = (value << 16) | (cls[1] << 8) | cls[0];
 
 	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
+		int err = -errno;
 		error("Can't write class of device: %s (%d)",
 							strerror(err), err);
-		return -err;
-	}
-
-	return 0;
-}
-
-int set_major_class(int dd, const uint8_t *cls, uint8_t major)
-{
-	uint32_t dev_class;
-
-	dev_class = (cls[2] << 16) | ((cls[1] & 0x20) << 8) |
-						((major & 0xdf) << 8) | 0x00;
-
-	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
-		error("Can't write class of device: %s (%d)",
-							strerror(err), err);
-		return -err;
-	}
-
-	return 0;
-}
-
-int set_minor_class(int dd, const uint8_t *cls, uint8_t minor)
-{
-	uint32_t dev_class;
-
-	dev_class = (cls[2] << 16) | (cls[1] << 8) | minor;
-
-	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
-		error("Can't write class of device: %s (%d)",
-							strerror(err), err);
-		return -err;
+		return err;
 	}
 
 	return 0;
@@ -1326,10 +1291,10 @@ int set_major_and_minor_class(int dd, const uint8_t *cls,
 						((major & 0xdf) << 8) | minor;
 
 	if (hci_write_class_of_dev(dd, dev_class, HCI_REQ_TIMEOUT) < 0) {
-		int err = errno;
+		int err = -errno;
 		error("Can't write class of device: %s (%d)",
 							strerror(err), err);
-		return -err;
+		return err;
 	}
 
 	return 0;
@@ -1423,10 +1388,7 @@ void hcid_dbus_setscan_enable_complete(bdaddr_t *local)
 		goto failed;
 	}
 
-	adapter_remove_discov_timeout(adapter);
-
-	if (adapter_get_scan_mode(adapter) != rp.enable)
-		adapter_mode_changed(adapter, rp.enable);
+	adapter_mode_changed(adapter, rp.enable);
 
 failed:
 	if (dd >= 0)
@@ -1520,6 +1482,10 @@ int hcid_dbus_get_io_cap(bdaddr_t *local, bdaddr_t *remote,
 	if (get_auth_requirements(local, remote, auth) < 0)
 		return -1;
 
+	if (!adapter_pairing_initiator(adapter, remote) &&
+			!adapter_is_pairable(adapter))
+		return -EPERM;
+
 	ba2str(remote, addr);
 
 	/* For CreatePairedDevice use dedicated bonding */
