@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2006-2007  Nokia Corporation
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -29,8 +29,11 @@ struct btd_device *device_create(DBusConnection *conn, struct btd_adapter *adapt
 void device_remove(DBusConnection *conn, struct btd_device *device);
 gint device_address_cmp(struct btd_device *device, const gchar *address);
 int device_browse(struct btd_device *device, DBusConnection *conn,
-			DBusMessage *msg, uuid_t *search);
-void device_probe_drivers(struct btd_device *device, GSList *uuids, sdp_list_t *recs);
+			DBusMessage *msg, uuid_t *search, gboolean reverse);
+void device_probe_drivers(struct btd_device *device, GSList *profiles);
+const sdp_record_t *btd_device_get_record(struct btd_device *device,
+						const char *uuid);
+void btd_device_add_uuid(struct btd_device *device, const char *uuid);
 struct btd_adapter *device_get_adapter(struct btd_device *device);
 void device_get_address(struct btd_device *adapter, bdaddr_t *bdaddr);
 const gchar *device_get_path(struct btd_device *device);
@@ -42,7 +45,7 @@ void device_set_temporary(struct btd_device *device, gboolean temporary);
 void device_set_cap(struct btd_device *device, uint8_t cap);
 void device_set_auth(struct btd_device *device, uint8_t auth);
 uint8_t device_get_auth(struct btd_device *device);
-gboolean device_set_paired(DBusConnection *conn, struct btd_device *device,
+int device_set_paired(DBusConnection *conn, struct btd_device *device,
 			struct bonding_request_info *bonding);
 gboolean device_get_connected(struct btd_device *device);
 void device_set_connected(DBusConnection *conn, struct btd_device *device,
@@ -54,7 +57,7 @@ void device_set_secmode3_conn(struct btd_device *device, gboolean enable);
 struct btd_device_driver {
 	const char *name;
 	const char **uuids;
-	int (*probe) (struct btd_device *device, GSList *records);
+	int (*probe) (struct btd_device *device, GSList *uuids);
 	void (*remove) (struct btd_device *device);
 };
 
